/*

Dynaform Service, exposing 8 public methods
===========================================

build(model, meta) - takes a model and (lazy)metadata and returns an object { form: FormGroup, meta: ModeledMetaData }
autoBuildFormGroupAndMeta(model, meta) - synonym for build
autoBuildModeledMeta(model, meta) - takes a model and (lazy)metadata and returns expanded metadata

buildFormGroup(metadata) - builds FormGroups from modelled metdata, recursively if necessary
buildFieldSpecificMeta(metadata) - use field metadta models to fill out metadata
combineModelWithMeta(model, extraMeta) - automatically generated metadata for model then combines extra metadata
combineExtraMeta(metadata, extraMeta) - combine extra metadata into metatdata, lazyly and recursively
autoMeta(model) - generate basic metadata from a raw or mapped model, recursively if necessary


NOTES
-----
This class acts as an injectable wraper around the exports of meta-utils.ts,
as well as creating a buildFormGroup function using the injected FormBuilder singleton


USAGE
-----

TO ADD ...


EXAMPLES
--------

TO ADD ...

*/

import { Injectable } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import {
	autoMeta, combineModelWithMeta, combineExtraMeta,
	buildFieldSpecificMeta, buildFormGroupFunctionFactory
} from './meta-utils';

export interface FormAndMeta {
	form: FormGroup;
	meta: StringMap;
}


@Injectable()
export class DynaformService {

	public buildFormGroup: (meta) => FormGroup;

	constructor(private fb: FormBuilder) {
		this.buildFormGroup = buildFormGroupFunctionFactory(fb);
	}

	build(model: StringMap, meta = {}, createFromMeta = false): FormAndMeta {
		// Short name for autoBuildFormGroupAndMeta
		return this.autoBuildFormGroupAndMeta(model, meta, createFromMeta);
	}

	// -----------------------------------------------------------------------------------------------------------------
	// Convenience methods combining several steps

	autoBuildFormGroupAndMeta(model: StringMap, meta = {}, createFromMeta = false): FormAndMeta {
		if (Object.keys(model).length === 0) {
			createFromMeta = true;
		}
		const modelWithMeta = this.autoBuildModeledMeta(model, meta, createFromMeta);
		return {
			form: this.buildFormGroup(modelWithMeta),
			meta: modelWithMeta
		};
	}

	autoBuildModeledMeta(model: StringMap, meta = {}, createFromMeta = false) {
		const modelWithMeta = this.combineModelWithMeta(model, meta, createFromMeta);
		return this.buildFieldSpecificMeta(modelWithMeta);
	}

	// -----------------------------------------------------------------------------------------------------------------
	// Build field-type-specific metadata using the form field models (see dynaform/models)

	buildFieldSpecificMeta(meta) {
		return buildFieldSpecificMeta(meta);
	}

	// -----------------------------------------------------------------------------------------------------------------
	// Lower-level methods

	combineModelWithMeta(model: StringMap, meta, createFromMeta = false) {
		return combineModelWithMeta(model, meta, createFromMeta);
	}

	combineExtraMeta(meta, extraMeta, createFromExtra = false) {
		return combineExtraMeta(meta, extraMeta, createFromExtra);
	}

	autoMeta(model) {
		return autoMeta(model);
	}

}

